;;  Diodentester mit RS232 Anschluss
;;  GP 5 = Serielle RS232 Data zum PC
;;  GP 0 = Eingang Polaritaet Testspannung
;;  AN 1 = Messeingang bei negativer Testspannung
;;  AN 2 = Messeingang bei positiver Testspannung

;RS232 Protokoll sieht folgendermassen aus (verwendet negative Logik!):
;Startbit: 		Low (5V)
;8 Datenbits:	Low (5V), High (0V), LSB wird zuerst gesendet
;Stopbit:		High (0V)
;Baudrate:	110 baud (9091us Pulsdauer)
;No Parity

;110baud wurd gewaehlt, da diese Rate auch fuer die Testspannung gilt und damit genug Aufladezeit fuer AD-s&h gibt

	processor  12F683
	radix  DEC
	__CONFIG 0x00C4

INDF    EQU 0x00
TMR0    EQU 0x01
STATUS  EQU 0x03
FSR     EQU 0x04
GPIO    EQU 0x05
INTCON  EQU 0x0B
PIR1	EQU	0x0C
TMR1L	EQU	0x0E
TMR1H	EQU	0x0F
T1CON	EQU	0x10
T2CON	EQU	0x12
CCPRL	EQU	0x13
CCPRH	EQU	0x14
CCPCON	EQU	0x15
WDTCON	EQU	0x18
CMCON	EQU	0x19
CMCON1	EQU	0x1A
ADRESH	EQU	0x1E
ADCON0	EQU	0x1F
OPTREG	EQU	0x81
TRISIO  EQU 0x85
PIE1	EQU	0x8C
PCON	EQU	0x8E
OSCCON	EQU	0x8F
OSCTUN	EQU	0x90
PR2		EQU	0x92
WPU		EQU	0x95
IOC		EQU	0x96
VRCON	EQU	0x99
EEDATA	EQU	0x9A
EEADR	EQU	0x9B
EECON1	EQU	0x9C
EECON2	EQU	0x9D
ANSEL	EQU	0x9F

_CARRY  EQU 0
_ZERO   EQU 2
_BANK   EQU 5
_RSDAT	EQU	5
_POLIN  EQU 0
_START  EQU 1

MEM		EQU	0x20
n		EQU	   MEM		;allgemenier Zaehler
z		EQU	   MEM+1	;Zaehler fuer Warteschleifen
TMP		EQU	   MEM+2	;Temp-Variable fuer RS232-Routine

Erg		EQU	  MEM+11	;Pointer auf Low-Byte des Ergebnisses hoeherer Kalkulationen
Pr1	    EQU   MEM+12	;Pointer auf Low-Byte des 1. Operator in hoeheren kalkulationen
Pr2	    EQU   MEM+13	;Pointer auf Low-Byte des 2. Operator in hoeheren kalkulationen

AL	    EQU   MEM+17	;Zwischenergebnis in den Messungskalkulationen
AM	    EQU   MEM+18	;Zwischenergebnis in den Messungskalkulationen
AH	    EQU   MEM+19	;Zwischenergebnis in den Messungskalkulationen
BL	    EQU   MEM+20	;Zwischenergebnis in den Messungskalkulationen
BM	    EQU   MEM+21	;Zwischenergebnis in den Messungskalkulationen
BH	    EQU   MEM+22	;Zwischenergebnis in den Messungskalkulationen
CL	    EQU   MEM+23	;Zwischenergebnis in den Messungskalkulationen
CM	    EQU   MEM+24	;Zwischenergebnis in den Messungskalkulationen
CH	    EQU   MEM+25	;Zwischenergebnis in den Messungskalkulationen
Z1	    EQU   MEM+26	;Zwischen-Ergenis in den hoeheren Kalkulationen
Z2	    EQU   MEM+27	;Zwischen-Ergenis in den hoeheren Kalkulationen
Z3	    EQU   MEM+28	;Zwischen-Ergenis in den hoeheren Kalkulationen
	


		ORG		0
		GOTO  	main

		;Warteschleife fuer Baudrate 110 (=9091us)
		;10 Zyklen werden schon in der RS232 Routine zwischen jedes Bit gesetzt
delay11	MOVLW  	.220				;zusammen mit dem aufrufenden CALL
		MOVWF  	TMR1H				;werden hier 9081 Zyklen gewartet
		MOVLW	.149				;2+6+9067+6 (Call + Zaehlervorbereitung + Zaehler +Austritt)
		MOVWF	TMR1L
		CLRF	PIR1
		BSF		T1CON,0	
w1		BTFSS	PIR1,0	
		GOTO	w1	
		BCF		T1CON,0	
		CLRF	PIR1		
		RETURN

		;Warteschleife fuer halbe Baudrate 110 (=4545us)
delay22	MOVLW  	.238				;zusammen mit dem aufrufenden CALL
		MOVWF  	TMR1H				;werden hier 4545 Zyklen gewartet
		MOVLW	.77					;2+6+4531+6 (Call + Zaehlervorbereitung + Zaehler +Austritt)
		MOVWF	TMR1L
		CLRF	PIR1
		BSF		T1CON,0	
w2		BTFSS	PIR1,0	
		GOTO	w2	
		BCF		T1CON,0	
		CLRF	PIR1		
		RETURN

		;RS232 Routine, Sendet Byte in W
		;Jeder Wechsel zwischen zwei Bits dauert 10 Zyklen + 1 delay
Send	MOVWF	TMP				;zu sendendes Byte nach TMP
		MOVLW 	8
		MOVWF 	n				;8 Bits senden
		BSF   	GPIO, 5	     	;Startbit setzen (Low = 5V)
		NOP
		NOP
		NOP
		NOP
		NOP
		CALL  	delay11			;Baudzeit warten
Loop_S 	RRF   	TMP,1			;Bits durchschieben
		BTFSS 	STATUS, _CARRY	;Bit checken = 1?
		GOTO	S0				;nein: Sende 0
		NOP						;ja: Sende 1
		BCF	  	GPIO, 5			;High=0V
		GOTO	Paus			;Ab zur Pause 
S0		BSF   	GPIO, 5			;Low=5V
		NOP
		NOP
Paus	CALL  	delay11			;Baudzeit warten
		DECFSZ 	n,1				;n=n-1, n=0 (Byte zu Ende)?
		GOTO  	Loop_S			;nein: weiter mit naechstem Bit
		NOP						;ja: Stopbit setzen
		NOP
		NOP
		NOP
		NOP
		BCF   	GPIO, 5			;Stopbit setzen (High = 0V)
		CALL  	delay11
		CALL	delay11
		RETURN





;******* Addit *******************************************************
Addit							;Addiert 24bit mit 24bit zu 24bit Ergebnis. Pr1, Pr2 zeigen auf Summanden, Erg auf Ergebnis
		MOVF	Pr1,W			;Pr1, Pr2 bleiben unveraendert, Erg hat danach Ergebnis
		ADDLW	.2
		MOVWF	Pr1				;Pr1 zeigt auf Pr1high
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z1				;Z1 hat Wert von Pr1high
		MOVF	Pr2,W
		ADDLW	.2
		MOVWF	Pr2				;Pr2 zeigt auf Pr2high
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z2				;Z2 hat Wert von Pr2high
		MOVF	Erg,W
		ADDLW	.2
		MOVWF	Erg				;Erg zeigt auf Erghigh
		MOVWF   FSR
		MOVF	Z1,W
		ADDWF	Z2,W			;W=Z1+Z2
		MOVWF	INDF			;Erg=W
		DECF	Pr1,1			;Pointer zeigen auf Midbytes
		DECF	Pr2,1
		DECF	Erg,1
		MOVF	Pr1,W
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z1				;Z1 hat Wert von Pr1mid
		MOVF	Pr2,W
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z2				;Z2 hat Wert von Pr2mid
		MOVF    Erg,W
		MOVWF   FSR
		MOVF	Z1,W
		ADDWF	Z2,W			;W=Z1+Z2
		MOVWF	INDF			;Erg=W
		BTFSS	STATUS,_CARRY	;Pruefe, ob Ueberlauf
		GOTO	Add1			;Kein Ueberlauf: weiter
		INCF	Erg,1			;Ueberlauf: Erg zeigt auf Highbyte
		MOVF	Erg,W
		MOVWF	FSR
		INCF	INDF,1			;Erhoehe Highbyte
		DECF	Erg,1			;Erg zeigt auf Midbyte
Add1	DECF	Pr1,1			;Pointer zeigen auf Lowbytes
		DECF	Pr2,1
		DECF	Erg,1
		MOVF	Pr1,W
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z1				;Z1 hat Wert von Pr1low
		MOVF	Pr2,W
		MOVWF	FSR
		MOVF	INDF,W
		MOVWF	Z2				;Z2 hat Wert von Pr2low
		MOVF    Erg,W
		MOVWF	FSR
		MOVF	Z1,W
		ADDWF	Z2,W			;W=Z1+Z2
		MOVWF	INDF			;Erg=W
		BTFSS	STATUS,_CARRY	;Pruefe, ob Ueberlauf
		GOTO	Adde			;Kein Ueberlauf: weiter
		INCF	Erg,1			;Ueberlauf: Erg zeigt auf Midbyte
		MOVF	Erg,W
		MOVWF	FSR
		INCF	INDF,1			;Erhoehe Midbyte
		BTFSS	STATUS,_ZERO	;Pruefe, ob Ueberlauf
		GOTO	Add2			;Kein Ueberlauf: weiter
		INCF	Erg,1			;Ueberlauf: Erg zeigt auf Highbyte
		MOVF	Erg,W
		MOVWF	FSR
		INCF	INDF,1			;Erhoehe Highbyte
		DECF	Erg,1			;Erg zeigt auf Midbyte
Add2	DECF	Erg,1			;Erg zeigt auf Lowbyte
Adde	RETURN



main						;Define configuration registers
		BCF   	STATUS,_BANK	;auf Bank 0 schalten
		CLRF  	TMR0			;01h:Timer zuruecksetzen auf Null
		CLRF 	STATUS			;03h: Status
		MOVLW 	0x07
		MOVWF 	CMCON			;07h: Comparator ausschalten	
		CLRF  	INTCON			;0Bh: INTCON: Interrupts ausschalten
		CLRF 	PIR1			;0Ch: PIR1: Alle Interrupts auf disabled
		CLRF 	T1CON			;10h: TIM1: Timer1 aus
		CLRF	T2CON
		CLRF	CCPRL
		CLRF	CCPRH
		CLRF	CCPCON
		CLRF	WDTCON
		CLRF	CMCON1
		MOVLW 	0x01
		MOVWF 	ADCON0			;1Fh: ADC einschalten	
		BSF   	STATUS,_BANK	;auf Bank 1 schalten
		MOVLW	0x67
		MOVWF	OSCCON
		CLRF	OSCTUN
		CLRF	PR2
		MOVLW 	0x1F
		MOVWF 	TRISIO			;85h: TRISIO: nur GP5 ist output
		CLRF 	PIE1			;8Ch: PIE1: Interrupts disablen
		CLRF 	IOC				;96h: IOC: Alle Interrupts disabled
		CLRF 	WPU				;95h: Pull-ups disablen
		CLRF 	VRCON			;99h: Comparator refVoltage disablen
		CLRF 	EECON1			;9Ch: EEPROM vorbereiten
		MOVLW 	0x56
		MOVWF 	ANSEL			;9Fh: ANSEL: Fosc/16, nur AN1 und AN2 = AnalogIn
		MOVLW 	0x80				
		MOVWF	OPTREG			;Option: pullup disabled, timer input intern, prescaler assigned to timer, prescale by 1:2	
		BCF   	STATUS,_BANK	;auf bank 0 schalten
		CLRF  	GPIO			;05h: PORTA: GP5 und GP4 auf 0 setzen

;Starte Hauptschleife (Endlos)
start	CLRF	AL
		CLRF	AM
		CLRF	AH
		CLRF	BL
		CLRF	BM
		CLRF	BH
wth1	BTFSS	GPIO, _POLIN	;Warte auf positives Bit (Startbit)
		GOTO	wth1
		MOVLW	0x04
		MOVWF	n		;Zaehler initialisieren
wtl		BTFSC	GPIO, _POLIN	;Warte auf negatives Bit
		GOTO	wtl		
		MOVLW	0x05		;Auf Kanal 1 schalten
		MOVWF	ADCON0
		CALL	delay22		;Warte halbe Signalperiode - fuer AD-s&h Zeit
		BSF   	ADCON0,_START		;Starte ADC Conversion
BV_1	BTFSC 	ADCON0,_START		
		GOTO  	BV_1			
		MOVF  	ADRESH,W		;ADRESH	
		MOVWF 	CL				;nach CL speichern
		CLRF 	CM
		CLRF 	CH
		MOVLW 	CL			
		MOVWF 	Pr1				;Pr1 zeigt auf Messwert
		MOVLW	AL
		MOVWF 	Pr2				;Pr2 zeigt auf A
		MOVLW 	AL
		MOVWF 	Erg				;Erg zeigt auf A
		CALL 	Addit			;A=A+Messwert
wth2	BTFSS	GPIO, _POLIN	;Warte auf positives Bit
		GOTO	wth2
		MOVLW	0x09		;Auf Kanal 2 schalten
		MOVWF	ADCON0
		CALL	delay22		;Warte halbe Signalperiode - fuer AD-s&h Zeit	
		BSF   	ADCON0,_START		;Starte ADC Conversion
BV_2	BTFSC 	ADCON0,_START		
		GOTO  	BV_2			
		MOVF  	ADRESH,W		;ADRESH
		MOVWF 	CL				;nach CL speichern
		CLRF 	CM
		CLRF 	CH
		MOVLW 	CL			
		MOVWF 	Pr1				;Pr1 zeigt auf Messwert
		MOVLW	BL
		MOVWF 	Pr2				;Pr2 zeigt auf B
		MOVLW 	BL
		MOVWF 	Erg				;Erg zeigt auf B
		CALL 	Addit			;B=B+Messwert

		DECFSZ	n,1			;4 Messwerte aufnehmen und aufaddieren
		GOTO	wtl			
		
		RRF		AM,1			;Aufaddierte Ergebnisse durch 4 mitteln
		RRF		AL,1	
		RRF		AM,1	
		RRF		AL,1	
		RRF		BM,1	
		RRF		BL,1			
		RRF		BM,1			
		RRF		BL,1			;8-Bit Ergebnisse liegen jetzt in AL und BL			

		MOVF	AL,0		;AL (-12V Messergebnis)
		CALL	Send		;senden
		MOVF	BL,0		;BL (+12V Messergebnis)
		CALL	Send		;senden

		GOTO 	start	
		END
